! Temperature for techBASIC
! Copyright (c) 2014 Micono Utilitites
!  v0.1 2014/1/13 by Michio Ono

!-------------------------
! Service / Characteristic UUID
!-------------------------
RFDUINO_SERVICE_UUID$="2220"
RFDUINO_RECIVE_UUID$="2221"
RFDUINO_SEND_UUID$="2222"
RFDUINO_DISCONNECT_UUID$="2223"
max_data=12

!------------------------
! Common Definitions
!------------------------
RFDUINO_SUCCESS=0
RFDUINO_FAILURE=-1
HIGH=1
LOW=0
TRUE=1
FALSE=0
YES=1
NO=0
ENABLE=1
DISABLE=0

!OUTPUT_PIN=1
!INPUT_PIN=0
!PULLUP=1
!NO_PULLS=0

!------------------------------------
! Constant, Variable, GUI, etc. Several Data
!------------------------------------
! Array is 1Base

! Connect Status
Dim isScanning As Integer
Dim isConnected As Integer
Dim isReady As Integer

!--------------------------
! BLE variables
!--------------------------

! Pass an empty array to get all available devices.
DIM uuid(1) AS STRING
DIM BleDEV AS BLEPeripheral

! Characteristic
DIM sendChar AS BLECharacteristic
DIM reciveChar AS BLECharacteristic
DIM disconnectChar AS BLECharacteristic

Dim foundBLE(3) AS BLEPeripheral
Dim foundNum as Integer
Dim scanTime as Long
scanWait=2

Dim foundTxP(3) As Integer
Dim txPower As Integer

!*****************************************************
! GUI
!*****************************************************
height = Graphics.height
width = Graphics.width

! for LOG
Dim msg as Label
msg=graphics.newLabel(0,0,width,20)!height-20,width,20)
msg.setBackgroundColor(0.7,0.7,0.7)
msg.setAlignment(2)
msg.setText("RFDUINO test...")

ctw=150
cth=40

Dim cnct as Button
cnct=graphics.newButton(width/2,cth,ctw,40)
cnct.setbackgroundcolor(1,0,0)
cnct.setTitle("Connect")
cnct.setGradientColor(1,1,1)

Dim clbl as Label
clbl=graphics.newLabel(width/2,140,160,30)
clbl.setText("0 °C")
clbl.setFont("",32,0)
clbl.setColor(1,0,0)

Dim flbl as Label
flbl=graphics.newLabel(width/2,180,160,30)
flbl.setText("0 °F")
flbl.setFont("",32,0)
flbl.setColor(0,0,1)

Dim tmvw As ImageView
tmvw=graphics.newImageView(width/2-100,30,80,320)
tmvw.loadImage("thermometer.png",0) !http://guthub.com/RFduino/


!########################################
! Program Start!
!########################################

System.showGraphics(1,1)
System.clearConsole

BLEscanStart  ! BLE Scan start

!===============================
! Events and touches driven
!===============================
SUB nullEvent(time as double)
  if isConnected=NO and isScanning=YES and scantime>0 then BLEConnect
end SUB

SUB touchUpInside (ctrl AS Button, time AS DOUBLE)
  if isConnected=NO then
    if ctrl=cnct then
      BLEscanStart
    end if
  else if isReady then
    if ctrl=cnct then
      LogMsg("The connection was lost.")
      e=disconnectRFduino
      System.Wait(0.1)
      BLEdisconnect
    end if
  end if
END SUB


!===============================
Sub LogMsg(s as string)
  msg.setText(s)
  !print s
End Sub

!===============================
! Initialize
!===============================
Sub initializePreferVariables()
  foundNum=0
  foundTxP(1)=0
  foundTxP(2)=0
  foundTxP(3)=0
  txPower=0
  
  !Status
  isReady=NO
  isConnected=NO
  isScanning=NO
  
  BleDEV=NULL
End Sub

!===============================
Sub ConnectStatusIndicator()
  if isConnected=NO then
    cnct.setBackgroundcolor(1,0,0)
    cnct.setTitle("Connect")
    LogMsg("The connection was lost.")
  else
    if isReady then
     cnct.setBackgroundcolor(0,1,0)
     LogMsg("Ready for " & BleDEV.bleName)
   else
      cnct.setTitle("Disconnect")
   end if
  end if
End Sub

!+++++++++++++++++++++++
! RFduino API
!+++++++++++++++++++++++

!++++++++++++
! For Recive data
!++++++++++++

Sub didReceive(d() as Integer)
  Dim celsius As Single
  Dim fahrenh As Single
  celsius=dataSng(d)
  fahrenh=(celsius*9/5)+32
  print $ celsius$ using "##.## °C";celsius
  print $ fahrenh$ using "##.## °F";fahrenh
  clbl.setText(celsius$)
  flbl.setText(fahrenh$)
End Sub

Function dataStr(d() as Integer) As String
  Dim s As String
  for i=1 to UBOUND(d,1)
    s=s+chr(Asc(d(i)))
  next
  dataStr=s
End Function

Function dataByte(d() as Integer) As Integer
  dataByte=d(1) !Uint8
End Function

Function dataInt(d() as Integer) As Integer
  dataInt=Math.unpackInt(d)
End Function

Function dataLng(d() as Integer) As Long
  dataLng=Math.unpackInt(d)
End Function

Function dataSng(d() as Integer) As Single
 dataSng=Math.unpackSng(d)
End Function

Function dataDouble(d() as Integer) As Double
 dataDouble=Math.unpackDbl(d)
End Function

!++++++++++++
! For Send data
!++++++++++++

Function sendStr(d As String) As Integer
  if isReady then
  n=Len(d)
  Dim t(n) As Integer
  for i=1 to n
   t(i)=Asc(Mid(d,i,1))
  next
    BleDEV.writeCharacteristic(sendChar,t)
    sendStr=RFDUINO_SUCCESS
  else
    sendStr=RFDUINO_FAILURE
  end if
End Function

Function sendByte(d As Integer) As Integer
  if isReady then
    t=[1]
    t(1)=d BITAND $FF
    BleDEV.writeCharacteristic(sendChar,t)
    sendByte=RFDUINO_SUCCESS
  else
    sendByte=RFDUINO_FAILURE
  end if
End Function

Function sendInt(d As Integer) As Integer
  if isReady then
    t=[2]
    t=Math.packInt(d)
    BleDEV.writeCharacteristic(sendChar,t)
    sendInt=RFDUINO_SUCCESS
  else
    sendInt=RFDUINO_FAILURE
  end if
End Function

Function sendLng(d As Long) As Integer
  if isReady then
    t=[4]
    t=Math.packLng(d)
    BleDEV.writeCharacteristic(sendChar,t)
    sendLng=RFDUINO_SUCCESS
  else
    sendLng=RFDUINO_FAILURE
  end if
End Function

Function sendSng(d As Long) As Integer
  if isReady then
    t=[4]
    t=Math.packSng(d)
    BleDEV.writeCharacteristic(sendChar,t)
    sendSng=RFDUINO_SUCCESS
  else
    sendSng=RFDUINO_FAILURE
  end if
End Function

Function sendDbl(d As Long) As Integer
  if isReady then
    t=[8]
    t=Math.packDbl(d)
    BleDEV.writeCharacteristic(sendChar,t)
    sendDbl=RFDUINO_SUCCESS
  else
    sendDbl=RFDUINO_FAILURE
  end if
End Function


!++++++++++++
! Disconnect
!++++++++++++
Function disconnectRFduino() As Integer
  if isReady then
    t=[1]
    t(1)=1
    BleDEV.writeCharacteristic(disconnectChar,t)
    disconnectRFduino=RFDUINO_SUCCESS
  else
  disconnectRFduino=RFDUINO_FAILURE
  end if
End Function

!===================
!%%%%%%%%%%%%%%%%%%%
! Bluetooth LE
!%%%%%%%%%%%%%%%%%%%
!===================

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! Called when a peripheral is found. If it is a target device, we
! initiate a connection to is and stop scanning for peripherals.
!
! Parameters:
!    time - The time when the peripheral was discovered.
!    peripheral - The peripheral that was discovered.
!    services - List of services offered by the device.
!    advertisements - Advertisements (information provided by the
!        device without the need to read a service/characteristic)
!    rssi - Received Signal Strength Indicator
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SUB BLEDiscoveredPeripheral(time AS DOUBLE, peripheral AS BLEPeripheral,services() AS STRING, advertisements(,) AS STRING, rssi AS SINGLE)
  LogMsg("Found " & peripheral.bleName & " UUID = " & peripheral.uuid)
  if foundNum<3 then
    foundNum=foundNum+1
    foundBLE(foundNum)=peripheral
    for i=1 to Ubound(advertisements,1)
     if advertisements(i,1) = "kCBAdvDataTxPowerLevel" then
       foundTxP(foundNum)=Val(advertisements(i,2))
     end if
    next
    scanTime=system.ticks
  end if
END SUB

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! Called to report information about the connection status of the
! peripheral or to report that services have been discovered.
!
! Parameters:
!    time - The time when the information was received.
!    peripheral - The peripheral.
!    kind - The kind of call. One of
!        1 - Connection completed
!        2 - Connection failed
!        3 - Connection lost
!        4 - Services discovered
!    message - For errors, a human-readable error message.
!    err - If there was an error, the Apple error number. If there
!        was no error, this value is 0.
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SUB BLEPeripheralInfo (time AS DOUBLE, peripheral AS BLEPeripheral, kind AS INTEGER, msg AS STRING, err AS LONG)
  IF kind = 1 THEN
    LogMsg("Connected to " & peripheral.bleName)
    isConnected=YES !isConnected=BleDEV.IsConnected
    ConnectStatusIndicator
    peripheral.discoverServices(uuid)
    
  ELSE IF kind = 2 OR kind = 3 THEN
    LogMsg("The connection was lost.")
    isConnected=NO
    isReady=NO
    ConnectStatusIndicator
    
  ELSE IF kind = 4 THEN
    ! Services were found. If it is the main service, begin discovery
    ! of its characteristics.
    DIM availableServices(1) AS BLEService
    availableServices = peripheral.services
    FOR a = 1 TO UBOUND(availableServices, 1)
      LogMsg("Found service "& availableServices(a).UUID)
      IF availableServices(a).UUID = RFDUINO_SERVICE_UUID$ THEN
        peripheral.discoverCharacteristics(uuid, availableServices(a))
      END IF
    NEXT
  END IF
  
END SUB

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! Called to report information about a characteristic or included
! services for a service. If it is one we are interested in, start
! handling it.
!
! Parameters:
!    time - The time when the information was received.
!    peripheral - The peripheral.
!    service - The service whose characteristic or included
!        service was found.
!    kind - The kind of call. One of
!        1 - Characteristics found
!        2 - Included services found
!    message - For errors, a human-readable error message.
!    err - If there was an error, the Apple error number. If there
!        was no error, this value is 0.
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SUB BLEServiceInfo (time AS DOUBLE, peripheral AS BLEPeripheral, service AS BLEService, kind AS INTEGER, msg AS STRING, err AS LONG)
 IF kind = 1 THEN
  DIM characteristics(1) AS BLECharacteristic
  characteristics = service.characteristics
  FOR i = 1 TO UBOUND(characteristics, 1)
    LogMsg("Found characteristic " & str(i) & ": " & characteristics(i).uuid)
    If service.uuid=RFDUINO_SERVICE_UUID$ then
      Select Case characteristics(i).uuid
       Case RFDUINO_SEND_UUID$
         sendChar=characteristics(i)
       Case RFDUINO_RECIVE_UUID$
         reciveChar=characteristics(i)
         peripheral.setNotify(reciveChar,1)
       Case RFDUINO_DISCONNECT_UUID$
         disconnectChar=characteristics(i)
      End Select
   End if
  NEXT
  
  If service.uuid=RFDUINO_SERVICE_UUID$ then
    isReady=YES
    ConnectStatusIndicator
  end if
  
 END IF
END SUB

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! Called to return information from a characteristic.
!
! Parameters:
!    time - The time when the information was received.
!    peripheral - The peripheral.
!    characteristic - The characteristic whose information
!        changed.
!    kind - The kind of call. One of
!        1 - Called after a discoverDescriptors call.
!        2 - Called after a readCharacteristics call.
!        3 - Called to report status after a writeCharacteristics
!            call.
!    message - For errors, a human-readable error message.
!    err - If there was an error, the Apple error number. If there
!        was no error, this value is 0.
!
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SUB BLECharacteristicInfo (time AS DOUBLE, peripheral AS BLEPeripheral, characteristic AS BLECharacteristic, kind AS INTEGER, msg AS STRING, err AS LONG)
  IF kind = 2 THEN
    !LogMsg("Value for characteristic ") ! & characteristic.uuid " = ")
    DIM value(1) AS INTEGER
    value = characteristic.value
    SELECT CASE characteristic.uuid
      CASE RFDUINO_RECIVE_UUID$
        didReceive(value)
      CASE ELSE
        !PRINT "Unexpected UUID "; characteristic.uuid
      END SELECT
      
  ELSE IF kind = 3 THEN
  END IF
END SUB

!~~~~~~~~~~~~~~~~~~~
! Connect / Disconnect
!~~~~~~~~~~~~~~~~~~~

!~~~~~~~~~~~~~~~~~~~
Sub BLEscanStart()
  LogMsg("Start BLE Scanning...")
  isScanning=YES
  isConnected=NO
  isReady=NO
  foundNum=0
  scanTime=System.ticks
  BLE.startBLE
  BLE.startScan(uuid)
End Sub

!~~~~~~~~~~~~~~~~~~~
Sub BLEConnect()
  cnct.setbackgroundcolor(1,rnd(1),0)
  if system.ticks-scantime>scanWait then
    scantime=0
    LogMsg("Stopping Scan.")
    BLE.stopScan
    if foundNum>0 then
      Dim pn(3) as String
      for i=1 to foundnum
        pn(i)=foundBLE(i).bleName & " / " & foundBLE(i).uuid
      next
      n=foundnum
      if foundnum>1 then 
        n=graphics.showAlert("Connect","Select Peripheral","Cancel",_
        pn(1),pn(2),pn(3))-1
      end if
      Select Case n
        case 0:
          BLE.stopBLE
          ConnectStatusIndicator
        case else
          LogMsg("Connecting...")
          cnct.setbackgroundcolor(1,1,0)
          txPower=foundTxP(n)
          BleDEV = foundBLE(n)
          BLE.connect(BleDEV)
      End Select
    end if
  end if
End Sub

!~~~~~~~~~~~~~~~~~~~
Sub BLEdisconnect()
  BLE.disconnect(BleDEV)
  BLE.stopBLE
  initializePreferVariables
  ConnectStatusIndicator
End Sub

!---------------------------